% % =======================================================================
% % This function samples from the right truncated gamma distribution
% % with acceptance prob 0.9 using Phillippe (1997)
% % See: 
% % Chan, J. C. C. and Kroese, D. K. (2012). "Improved Cross-Entropy Method
% % for Estimation", Statistics and Computing, 22(5), 1031-1040.
% %
% % Please report any errors to joshuacc.chan@gmail.com
% % =======================================================================

function draws = rtgamrnd(a,b,t,N)
draws = zeros(N,1);
count=1;
p = 1.2;

n = ceil((p + sqrt(p^2 + 4*b*t))^2 /4);
wbar = ones(n,1);
wtilde = ones(n,1);
for k = 1 : n-1
    wbar(k+1) = wbar(k) * b * t / (a + k);
    wtilde(k+1) = wtilde(k) + wbar(k+1);
end
wtilde = wtilde/wtilde(end);
invM = sum((b*t).^(0:1:n-1) ./ gamma(1:1:n));

while count<=N
    k = find(wtilde>rand, 1 );
    x = betarnd(a,k);
    invrho = exp(b*t*x) * sum( (b*t*(1-x)).^(0:1:n-1) ./ gamma(1:1:n) );
    if rand*invrho < invM
        draws(count,1) = x*t;
        count = count + 1;
    end
end