% % =======================================================================
% % This code implements the VM method to compute the rare-event 
% % probability of large portfolio loss
% % See: 
% % Chan, J. C. C. and Kroese, D. K. (2012). "Improved Cross-Entropy Method
% % for Estimation", Statistics and Computing, 22(5), 1031-1040.
% %
% % Please report any errors to joshuacc.chan@gmail.com
% % =======================================================================

rand('state', sum(100*clock));
randn('state', sum(100*clock));
start_time = clock;

% find v_{vm}
Zmu = mean(sZ); Zvar = var(sZ);
etamu = mean(mean(seta));
temp = gamfit(sQ);
alpt = temp(1); lamt = 1/temp(2);
g = @(x) LR(x,[sZ sQ seta],nu,sig_eta);
vmpara = fminsearch(g,[Zmu Zvar alpt lamt etamu]);
Zmu = vmpara(1);
Zvar = vmpara(2); 
alpt = vmpara(3);
lamt = vmpara(4);
etamu = vmpara(5);

Z = Zmu + sqrt(Zvar)*randn(N,1);
eta = etamu + sqrt(sig_eta)*randn(N,n);
Q = gamrnd( alpt, 1/lamt, N, 1 );
sqQ = sqrt(Q);
X = (repmat(rho*Z,1,n) + sq1mrho2*eta) ./ repmat(sqQ,1,n);
L = a * sum(X>x,2);

c = (1-nu/2)*log(2) + nu/2*log(nu) - gammaln(nu/2) - log(2) - alpt*log(lamt) + gammaln(alpt);
w = log(Zvar)/2 - 1/2*(Z.^2 - (Z-Zmu).^2/Zvar);
w = w - 1/(2*sig_eta) * (2*sum(eta,2)*etamu - n*etamu^2);
w = w + c + (nu/2-1)*log(Q) - nu/2*Q - (alpt-1)*log(Q) + lamt*Q;
w = exp(w);
id = (L<=l);
w(id) = 0;
lhat = sum(w) / N;
RE = sqrt(var(w)/N) / lhat * 100;  % in percentage

disp( ['VM takes '  num2str( etime(clock, start_time) ) ' seconds' ] );
disp(' ' );
